@@ -6,6 +6,8 @@ import ctypes
 from abc import abstractmethod
 from typing import Any, Callable
 
+from packaging.version import parse as parse_version
+
 import awkward as ak
 from awkward._nplikes.cupy import Cupy
 from awkward._nplikes.jax import Jax
@@ -73,7 +75,7 @@ class NumpyKernel(BaseKernel):
             if numpy.is_own_array(x):
                 assert numpy.is_c_contiguous(x), "kernel expects contiguous array"
                 if x.ndim > 0:
-                    return ctypes.cast(x.ctypes.data, t)
+                    return ctypes.cast(numpy.memory_ptr(x), t)
                 else:
                     return x
             # Or, do we have a ctypes type
@@ -102,13 +104,25 @@ class JaxKernel(BaseKernel):
 
         self._jax = Jax.instance()
 
+        jax_module = ak.jax.import_jax()
+        self._ad_tracer_types = (jax_module._src.interpreters.ad.JVPTracer,)
+        if parse_version(jax_module.__version__) >= parse_version("0.7.0"):
+            self._ad_tracer_types += (jax_module._src.interpreters.ad.LinearizeTracer,)
+
     def _cast(self, x, t):
         if issubclass(t, ctypes._Pointer):
             # Do we have a JAX-owned array?
             if self._jax.is_own_array(x):
+                if self._jax.is_tracer_type(type(x)):
+                    # general message for any invalid JAX input type
+                    msg = f"Encountered {x} as an (invalid) input to the '{self._key[0]}' Awkward C++ kernel."
+                    # message specification for autodiff (i.e. when encountering a JVPTracer)
+                    if isinstance(x, self._ad_tracer_types):
+                        msg += " This kernel is not differentiable by the JAX backend."
+                    raise ValueError(msg)
                 assert self._jax.is_c_contiguous(x), "kernel expects contiguous array"
                 if x.ndim > 0:
-                    return ctypes.cast(x.unsafe_buffer_pointer(), t)
+                    return ctypes.cast(self._jax.memory_ptr(x), t)
                 else:
                     return x
             # Or, do we have a ctypes type
@@ -126,10 +140,9 @@ class JaxKernel(BaseKernel):
 
         args = materialize_if_virtual(*args)
 
-        if not any(Jax.is_tracer_type(type(arg)) for arg in args):
-            return self._impl(
-                *(self._cast(x, t) for x, t in zip(args, self._impl.argtypes))
-            )
+        return self._impl(
+            *(self._cast(x, t) for x, t in zip(args, self._impl.argtypes))
+        )
 
 
 class CupyKernel(BaseKernel):
