@@ -16,6 +16,7 @@ from awkward._nplikes.numpy_like import (
 )
 from awkward._nplikes.placeholder import PlaceholderArray
 from awkward._nplikes.shape import ShapeItem, unknown_length
+from awkward._nplikes.virtual import VirtualArray, materialize_if_virtual
 from awkward._typing import TYPE_CHECKING, Any, DType, Final, Literal, TypeVar, cast
 
 if TYPE_CHECKING:
@@ -39,6 +40,12 @@ ArrayLikeT = TypeVar("ArrayLikeT", bound=ArrayLike)
 
 
 class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
+    """
+    Some methods maintain virtualness while others are required to materialize the array.
+    The `materialize_if_virtual` function is used for that purpose.
+    If the input is not a virtual array, it's a zero cost operation.
+    """
+
     known_data: Final[bool] = True
     _module: Any
 
@@ -53,11 +60,25 @@ class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
         *,
         dtype: DTypeLike | None = None,
         copy: bool | None = None,
-    ) -> ArrayLikeT | PlaceholderArray:
+    ) -> ArrayLikeT | PlaceholderArray | VirtualArray:
         if isinstance(obj, PlaceholderArray):
             assert obj.dtype == dtype or dtype is None
             return obj
-        elif copy:
+        if isinstance(obj, VirtualArray):
+            if obj.is_materialized:
+                obj = obj.materialize()
+            else:
+                if obj.dtype == dtype or dtype is None:
+                    return obj
+                else:
+                    return VirtualArray(
+                        obj._nplike,
+                        obj._shape,
+                        dtype,
+                        lambda: self.asarray(obj.materialize(), dtype=dtype),
+                        lambda: obj.shape,
+                    )
+        if copy:
             return self._module.array(obj, dtype=dtype, copy=True)
         elif copy is None:
             return self._module.asarray(obj, dtype=dtype)
@@ -71,9 +92,20 @@ class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
 
     def ascontiguousarray(
         self, x: ArrayLikeT | PlaceholderArray
-    ) -> ArrayLikeT | PlaceholderArray:
+    ) -> ArrayLikeT | PlaceholderArray | VirtualArray:
         if isinstance(x, PlaceholderArray):
             return x
+        elif isinstance(x, VirtualArray):
+            if x.is_materialized:
+                return self.ascontiguousarray(x.materialize())  #  type: ignore[arg-type]
+            else:
+                return VirtualArray(
+                    x._nplike,
+                    x._shape,
+                    x._dtype,
+                    lambda: self.ascontiguousarray(x.materialize()),  # type: ignore[arg-type]
+                    lambda: x.shape,
+                )
         else:
             return self._module.ascontiguousarray(x)
 
@@ -82,6 +114,8 @@ class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
     ) -> ArrayLikeT:
         if isinstance(buffer, PlaceholderArray):
             raise TypeError("placeholder arrays are not supported in `frombuffer`")
+        if isinstance(buffer, VirtualArray):
+            raise TypeError("virtual arrays are not supported in `frombuffer`")
         return self._module.frombuffer(buffer, dtype=dtype, count=count)
 
     def from_dlpack(self, x: Any) -> ArrayLikeT:
@@ -123,7 +157,7 @@ class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
     def zeros_like(
         self, x: ArrayLikeT | PlaceholderArray, *, dtype: DTypeLike | None = None
     ) -> ArrayLikeT:
-        if isinstance(x, PlaceholderArray):
+        if isinstance(x, (PlaceholderArray, VirtualArray)):
             return self.zeros(x.shape, dtype=dtype or x.dtype)
         else:
             return self._module.zeros_like(x, dtype=dtype)
@@ -131,7 +165,7 @@ class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
     def ones_like(
         self, x: ArrayLikeT | PlaceholderArray, *, dtype: DTypeLike | None = None
     ) -> ArrayLikeT:
-        if isinstance(x, PlaceholderArray):
+        if isinstance(x, (PlaceholderArray, VirtualArray)):
             return self.ones(x.shape, dtype=dtype or x.dtype)
         else:
             return self._module.ones_like(x, dtype=dtype)
@@ -143,7 +177,7 @@ class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
         *,
         dtype: DTypeLike | None = None,
     ) -> ArrayLikeT:
-        if isinstance(x, PlaceholderArray):
+        if isinstance(x, (PlaceholderArray, VirtualArray)):
             return self.full(x.shape, fill_value, dtype=dtype or x.dtype)
         else:
             return self._module.full_like(
@@ -158,6 +192,7 @@ class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
         *,
         dtype: DTypeLike | None = None,
     ) -> ArrayLikeT:
+        start, stop, step = materialize_if_virtual(start, stop, step)
         assert not isinstance(start, PlaceholderArray)
         assert not isinstance(stop, PlaceholderArray)
         assert not isinstance(step, PlaceholderArray)
@@ -166,13 +201,16 @@ class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
     def meshgrid(
         self, *arrays: ArrayLikeT, indexing: Literal["xy", "ij"] = "xy"
     ) -> list[ArrayLikeT]:
-        return self._module.meshgrid(*arrays, indexing=indexing)
+        return self._module.meshgrid(
+            *materialize_if_virtual(*arrays), indexing=indexing
+        )
 
     ############################ testing
 
     def array_equal(
         self, x1: ArrayLikeT, x2: ArrayLikeT, *, equal_nan: bool = False
     ) -> bool:
+        x1, x2 = materialize_if_virtual(x1, x2)
         assert not isinstance(x1, PlaceholderArray)
         assert not isinstance(x2, PlaceholderArray)
         if equal_nan:
@@ -193,6 +231,7 @@ class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
         side: Literal["left", "right"] = "left",
         sorter: ArrayLikeT | None = None,
     ) -> ArrayLikeT:
+        x, values, sorter = materialize_if_virtual(x, values, sorter)
         assert not isinstance(x, PlaceholderArray)
         assert not isinstance(values, PlaceholderArray)
         assert not isinstance(sorter, PlaceholderArray)
@@ -209,6 +248,8 @@ class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
         if method != "__call__" or len(args) == 0:
             raise NotImplementedError
 
+        args = list(materialize_if_virtual(*args))
+
         if hasattr(ufunc, "resolve_dtypes"):
             return self._apply_ufunc_nep_50(ufunc, method, args, kwargs)
         else:
@@ -277,6 +318,7 @@ class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
         return impl(*non_generic_value_promoted_args, **(kwargs or {}))
 
     def broadcast_arrays(self, *arrays: ArrayLikeT) -> list[ArrayLikeT]:
+        arrays = materialize_if_virtual(*arrays)
         assert not any(isinstance(x, PlaceholderArray) for x in arrays)
         return self._module.broadcast_arrays(*arrays)
 
@@ -303,10 +345,22 @@ class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
         shape: tuple[ShapeItem, ...],
         *,
         copy: bool | None = None,
-    ) -> ArrayLikeT | PlaceholderArray:
+    ) -> ArrayLikeT | PlaceholderArray | VirtualArray:
         if isinstance(x, PlaceholderArray):
             next_shape = self._compute_compatible_shape(shape, x.shape)
             return PlaceholderArray(self, next_shape, x.dtype, x._field_path)
+        if isinstance(x, VirtualArray):
+            if not x.is_materialized:
+                next_shape = self._compute_compatible_shape(shape, x.shape)
+                return VirtualArray(
+                    self,
+                    next_shape,
+                    x.dtype,
+                    lambda: self.reshape(x.materialize(), next_shape),  # type: ignore[union-attr]
+                    lambda: next_shape,
+                )
+            else:
+                x = x.materialize()  # type: ignore[assignment]
 
         if copy is None:
             return self._module.reshape(x, shape)
@@ -383,18 +437,22 @@ class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
             raise IndexError(f"index value out of bounds (0, {length}): {index}")
 
     def nonzero(self, x: ArrayLikeT) -> tuple[ArrayLikeT, ...]:
+        (x,) = materialize_if_virtual(x)
         assert not isinstance(x, PlaceholderArray)
         return self._module.nonzero(x)
 
     def where(
         self, condition: ArrayLikeT, x1: ArrayLikeT, x2: ArrayLikeT
     ) -> ArrayLikeT:
+        condition, x1, x2 = materialize_if_virtual(condition, x1, x2)
         assert not isinstance(condition, PlaceholderArray)
         assert not isinstance(x1, PlaceholderArray)
         assert not isinstance(x2, PlaceholderArray)
+
         return self._module.where(condition, x1, x2)
 
     def unique_values(self, x: ArrayLikeT) -> ArrayLikeT:
+        (x,) = materialize_if_virtual(x)
         assert not isinstance(x, PlaceholderArray)
         np_unique_accepts_equal_nan = (
             "equal_nan" in inspect.signature(self._module.unique).parameters
@@ -417,6 +475,7 @@ class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
             )
 
     def unique_all(self, x: ArrayLikeT) -> UniqueAllResult:
+        (x,) = materialize_if_virtual(x)
         assert not isinstance(x, PlaceholderArray)
         values, indices, inverse_indices, counts = self._module.unique(
             x, return_counts=True, return_index=True, return_inverse=True
@@ -434,6 +493,7 @@ class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
         descending: bool = False,
         stable: bool = True,
     ) -> ArrayLikeT:
+        (x,) = materialize_if_virtual(x)
         assert not isinstance(x, PlaceholderArray)
         # Note: this keyword argument is different, and the default is different.
         kind = "stable" if stable else "quicksort"
@@ -449,6 +509,7 @@ class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
         *,
         axis: int | None = 0,
     ) -> ArrayLikeT:
+        arrays = materialize_if_virtual(*arrays)
         assert not any(isinstance(x, PlaceholderArray) for x in arrays)
         if _nplike_concatenate_has_casting(self._module):
             return self._module.concatenate(arrays, axis=axis, casting="same_kind")
@@ -462,6 +523,7 @@ class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
         *,
         axis: int | None = None,
     ) -> ArrayLikeT:
+        x, repeats = materialize_if_virtual(x, repeats)
         assert not isinstance(x, PlaceholderArray)
         assert not isinstance(repeats, PlaceholderArray)
         return self._module.repeat(x, repeats=repeats, axis=axis)
@@ -472,6 +534,7 @@ class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
         *,
         axis: int = 0,
     ) -> ArrayLikeT:
+        arrays = materialize_if_virtual(*arrays)
         assert not any(isinstance(x, PlaceholderArray) for x in arrays)
         arrays = list(arrays)
         return self._module.stack(arrays, axis=axis)
@@ -483,6 +546,7 @@ class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
         axis: int | None = None,
         bitorder: Literal["big", "little"] = "big",
     ) -> ArrayLikeT:
+        (x,) = materialize_if_virtual(x)
         assert not isinstance(x, PlaceholderArray)
         return self._module.packbits(x, axis=axis, bitorder=bitorder)
 
@@ -494,14 +558,17 @@ class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
         count: int | None = None,
         bitorder: Literal["big", "little"] = "big",
     ) -> ArrayLikeT:
+        (x,) = materialize_if_virtual(x)
         assert not isinstance(x, PlaceholderArray)
         return self._module.unpackbits(x, axis=axis, count=count, bitorder=bitorder)
 
     def broadcast_to(self, x: ArrayLikeT, shape: tuple[ShapeItem, ...]) -> ArrayLikeT:
+        (x,) = materialize_if_virtual(x)
         assert not isinstance(x, PlaceholderArray)
         return self._module.broadcast_to(x, shape)
 
     def strides(self, x: ArrayLikeT | PlaceholderArray) -> tuple[ShapeItem, ...]:
+        (x,) = materialize_if_virtual(x)
         if isinstance(x, PlaceholderArray):
             # Assume contiguous
             strides: tuple[ShapeItem, ...] = (x.dtype.itemsize,)
@@ -516,6 +583,7 @@ class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
     def add(
         self, x1: ArrayLikeT, x2: ArrayLikeT, maybe_out: ArrayLikeT | None = None
     ) -> ArrayLikeT:
+        x1, x2 = materialize_if_virtual(x1, x2)
         assert not isinstance(x1, PlaceholderArray)
         assert not isinstance(x2, PlaceholderArray)
         return self._module.add(x1, x2, out=maybe_out)
@@ -523,6 +591,7 @@ class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
     def logical_or(
         self, x1: ArrayLikeT, x2: ArrayLikeT, *, maybe_out: ArrayLikeT | None = None
     ) -> ArrayLikeT:
+        x1, x2 = materialize_if_virtual(x1, x2)
         assert not isinstance(x1, PlaceholderArray)
         assert not isinstance(x2, PlaceholderArray)
         return self._module.logical_or(x1, x2, out=maybe_out)
@@ -530,6 +599,7 @@ class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
     def logical_and(
         self, x1: ArrayLikeT, x2: ArrayLikeT, *, maybe_out: ArrayLikeT | None = None
     ) -> ArrayLikeT:
+        x1, x2 = materialize_if_virtual(x1, x2)
         assert not isinstance(x1, PlaceholderArray)
         assert not isinstance(x2, PlaceholderArray)
         return self._module.logical_and(x1, x2, out=maybe_out)
@@ -537,20 +607,24 @@ class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
     def logical_not(
         self, x: ArrayLikeT, maybe_out: ArrayLikeT | None = None
     ) -> ArrayLikeT:
+        (x,) = materialize_if_virtual(x)
         assert not isinstance(x, PlaceholderArray)
         return self._module.logical_not(x, out=maybe_out)
 
     def sqrt(self, x: ArrayLikeT, maybe_out: ArrayLikeT | None = None) -> ArrayLikeT:
+        (x,) = materialize_if_virtual(x)
         assert not isinstance(x, PlaceholderArray)
         return self._module.sqrt(x, out=maybe_out)
 
     def exp(self, x: ArrayLikeT, maybe_out: ArrayLikeT | None = None) -> ArrayLikeT:
+        (x,) = materialize_if_virtual(x)
         assert not isinstance(x, PlaceholderArray)
         return self._module.exp(x, out=maybe_out)
 
     def divide(
         self, x1: ArrayLikeT, x2: ArrayLikeT, maybe_out: ArrayLikeT | None = None
     ) -> ArrayLikeT:
+        x1, x2 = materialize_if_virtual(x1, x2)
         assert not isinstance(x1, PlaceholderArray)
         assert not isinstance(x2, PlaceholderArray)
         return self._module.divide(x1, x2, out=maybe_out)
@@ -566,6 +640,7 @@ class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
         posinf: int | float | None = None,
         neginf: int | float | None = None,
     ) -> ArrayLikeT:
+        (x,) = materialize_if_virtual(x)
         assert not isinstance(x, PlaceholderArray)
         return self._module.nan_to_num(
             x, copy=copy, nan=nan, posinf=posinf, neginf=neginf
@@ -580,11 +655,13 @@ class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
         atol: float = 1e-8,
         equal_nan: bool = False,
     ) -> ArrayLikeT:
+        x1, x2 = materialize_if_virtual(x1, x2)
         assert not isinstance(x1, PlaceholderArray)
         assert not isinstance(x2, PlaceholderArray)
         return self._module.isclose(x1, x2, rtol=rtol, atol=atol, equal_nan=equal_nan)
 
     def isnan(self, x: ArrayLikeT) -> ArrayLikeT:
+        (x,) = materialize_if_virtual(x)
         assert not isinstance(x, PlaceholderArray)
         return self._module.isnan(x)
 
@@ -596,6 +673,7 @@ class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
         keepdims: bool = False,
         maybe_out: ArrayLikeT | None = None,
     ) -> ArrayLikeT:
+        (x,) = materialize_if_virtual(x)
         assert not isinstance(x, PlaceholderArray)
         return self._module.all(x, axis=axis, keepdims=keepdims, out=maybe_out)
 
@@ -607,6 +685,7 @@ class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
         keepdims: bool = False,
         maybe_out: ArrayLikeT | None = None,
     ) -> ArrayLikeT:
+        (x,) = materialize_if_virtual(x)
         assert not isinstance(x, PlaceholderArray)
         return self._module.any(x, axis=axis, keepdims=keepdims, out=maybe_out)
 
@@ -618,6 +697,7 @@ class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
         keepdims: bool = False,
         maybe_out: ArrayLikeT | None = None,
     ) -> ArrayLikeT:
+        (x,) = materialize_if_virtual(x)
         assert not isinstance(x, PlaceholderArray)
         return self._module.min(x, axis=axis, keepdims=keepdims, out=maybe_out)
 
@@ -629,12 +709,14 @@ class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
         keepdims: bool = False,
         maybe_out: ArrayLikeT | None = None,
     ) -> ArrayLikeT:
+        (x,) = materialize_if_virtual(x)
         assert not isinstance(x, PlaceholderArray)
         return self._module.max(x, axis=axis, keepdims=keepdims, out=maybe_out)
 
     def count_nonzero(
         self, x: ArrayLikeT, *, axis: ShapeItem | tuple[ShapeItem, ...] | None = None
     ) -> ArrayLikeT:
+        (x,) = materialize_if_virtual(x)
         assert not isinstance(x, PlaceholderArray)
         assert isinstance(axis, int) or axis is None
         return self._module.count_nonzero(x, axis=axis)
@@ -646,26 +728,31 @@ class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
         axis: int | None = None,
         maybe_out: ArrayLikeT | None = None,
     ) -> ArrayLikeT:
+        (x,) = materialize_if_virtual(x)
         assert not isinstance(x, PlaceholderArray)
         return self._module.cumsum(x, axis=axis, out=maybe_out)
 
     def real(self, x: ArrayLikeT) -> ArrayLikeT:
+        (x,) = materialize_if_virtual(x)
         assert not isinstance(x, PlaceholderArray)
         xr = self._module.real(x)
         # For numpy, xr is a view on x, but we don't want to mutate x.
         return self._module.copy(xr)
 
     def imag(self, x: ArrayLikeT) -> ArrayLikeT:
+        (x,) = materialize_if_virtual(x)
         assert not isinstance(x, PlaceholderArray)
         xr = self._module.imag(x)
         # For numpy, xr is a view on x, but we don't want to mutate x.
         return self._module.copy(xr)
 
     def angle(self, x: ArrayLikeT, deg: bool = False) -> ArrayLikeT:
+        (x,) = materialize_if_virtual(x)
         assert not isinstance(x, PlaceholderArray)
         return self._module.angle(x, deg)
 
     def round(self, x: ArrayLikeT, decimals: int = 0) -> ArrayLikeT:
+        (x,) = materialize_if_virtual(x)
         assert not isinstance(x, PlaceholderArray)
         return self._module.round(x, decimals=decimals)
 
@@ -677,6 +764,9 @@ class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
         precision: int | None = None,
         suppress_small: bool | None = None,
     ):
+        if isinstance(x, VirtualArray) and not x.is_materialized:
+            return "[## ... ##]"
+        (x,) = materialize_if_virtual(x)
         if isinstance(x, PlaceholderArray):
             return "[## ... ##]"
         return self._module.array_str(
@@ -689,6 +779,7 @@ class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
     def astype(
         self, x: ArrayLikeT, dtype: DTypeLike, *, copy: bool | None = True
     ) -> ArrayLikeT:
+        (x,) = materialize_if_virtual(x)
         assert not isinstance(x, PlaceholderArray)
         return x.astype(dtype, copy=copy)  # type: ignore[attr-defined]
 
@@ -699,4 +790,16 @@ class ArrayModuleNumpyLike(NumpyLike[ArrayLikeT]):
 
     @classmethod
     def is_own_array(cls, obj) -> bool:
+        # This kinda breaks the type(obj) -> ownership paradigm, but I don't see a simpler way for virtual arrays.
+        # We are required to know if different nplikes own virtual arrays throughout the code
+        # and that can only be determined by the underlying nplike of the virtual array
+        # as virtual arrays can generate either numpy or cupy ndarrays.
+        # The VirtualArray type is now enough. We need the underlying nplike to determine ownership.
+        # All nplikes implement an ndarray property so we can can use that.
+        # There is an extra isinstance check here but that has to live somewhere in the code either way to determine ownership.
+        if isinstance(obj, VirtualArray):
+            return cls.is_own_array_type(obj.nplike.ndarray)
         return cls.is_own_array_type(type(obj))
+
+    def memory_ptr(self, x: ArrayLikeT) -> int:
+        raise NotImplementedError()
