@@ -13,6 +13,7 @@ from awkward._layout import maybe_posaxis
 from awkward._meta.unionmeta import UnionMeta
 from awkward._nplikes.array_like import ArrayLike
 from awkward._nplikes.cupy import Cupy
+from awkward._nplikes.jax import Jax
 from awkward._nplikes.numpy import Numpy
 from awkward._nplikes.numpy_like import IndexType, NumpyMetadata
 from awkward._nplikes.placeholder import PlaceholderArray
@@ -169,7 +170,7 @@ class UnionArray(UnionMeta[Content], Content):
                 )
 
         if (
-            backend.index_nplike.known_data
+            backend.nplike.known_data
             and tags.length is not unknown_length
             and index.length is not unknown_length
             and tags.length > index.length
@@ -178,8 +179,8 @@ class UnionArray(UnionMeta[Content], Content):
                 f"{type(self).__name__} len(tags) ({tags.length}) must be <= len(index) ({index.length})"
             )
 
-        assert tags.nplike is backend.index_nplike
-        assert index.nplike is backend.index_nplike
+        assert tags.nplike is backend.nplike
+        assert index.nplike is backend.nplike
 
         self._tags = tags
         self._index = index
@@ -253,8 +254,8 @@ class UnionArray(UnionMeta[Content], Content):
             raise ValueError("invalid UnionArray: len(index) < len(tags)")
 
         length = self_tags.length
-        tags = ak.index.Index8.empty(length, backend.index_nplike)
-        index = ak.index.Index64.empty(length, backend.index_nplike)
+        tags = ak.index.Index8.empty(length, backend.nplike)
+        index = ak.index.Index64.empty(length, backend.nplike)
         contents = []
 
         # For each outer union content
@@ -419,14 +420,27 @@ class UnionArray(UnionMeta[Content], Content):
             ):
                 # only want to affect the part of the UnionArray.index for this tag
                 selection = tags.data == tag
-                # function-composition of this part of the UnionArray.index with the IndexedArray.index
-                index.data[selection] = content.index.data[index.data[selection]]
-                # now we don't have an IndexedArray anymore, but we want to preserve its parameters
-                contents[tag] = content.content.copy(
-                    parameters=parameters_union(
-                        content.content.parameters, content.parameters
+
+                if isinstance(backend.nplike, Jax):
+                    # function-composition of this part of the UnionArray.index with the IndexedArray.index
+                    index._data = content.index.data.at[selection].set(
+                        content.index.data[index.data[selection]]
+                    )
+                    # now we don't have an IndexedArray anymore, but we want to preserve its parameters
+                    contents[tag] = content.content.copy(
+                        parameters=parameters_union(
+                            content.content.parameters, content.parameters
+                        )
+                    )
+                else:
+                    # function-composition of this part of the UnionArray.index with the IndexedArray.index
+                    index.data[selection] = content.index.data[index.data[selection]]
+                    # now we don't have an IndexedArray anymore, but we want to preserve its parameters
+                    contents[tag] = content.content.copy(
+                        parameters=parameters_union(
+                            content.content.parameters, content.parameters
+                        )
                     )
-                )
 
         # If any contents are options, ensure all contents are options!
         if any(c.is_option for c in contents):
@@ -482,10 +496,10 @@ class UnionArray(UnionMeta[Content], Content):
         key1 = getkey(self, form, "tags")
         key2 = getkey(self, form, "index")
         container[key1] = ak._util.native_to_byteorder(
-            self._tags.raw(backend.index_nplike), byteorder
+            self._tags.raw(backend.nplike), byteorder
         )
         container[key2] = ak._util.native_to_byteorder(
-            self._index.raw(backend.index_nplike), byteorder
+            self._index.raw(backend.nplike), byteorder
         )
         for i, content in enumerate(self._contents):
             content._to_buffers(form.content(i), getkey, container, backend, byteorder)
@@ -523,7 +537,7 @@ class UnionArray(UnionMeta[Content], Content):
 
     def _repr(self, indent, pre, post):
         out = [indent, pre, "<UnionArray len="]
-        out.append(repr(str(self.length)))
+        out.append(repr(str(ak._util.maybe_length_of(self))))
         out.append(">")
         out.extend(self._repr_extra(indent + "    "))
         out.append("\n")
@@ -643,7 +657,7 @@ class UnionArray(UnionMeta[Content], Content):
             contents = []
             for tag, content in enumerate(self._contents):
                 if tag == tag_for_missing:
-                    indexedoption_index = self._backend.index_nplike.arange(
+                    indexedoption_index = self._backend.nplike.arange(
                         content.length + 1, dtype=np.int64
                     )
                     contents.append(
@@ -674,10 +688,10 @@ class UnionArray(UnionMeta[Content], Content):
 
             else:
                 # UnionArray is empty, so
-                nexttags = self._backend.index_nplike.full(
+                nexttags = self._backend.nplike.full(
                     len(carry_data), tag_for_missing, dtype=self._tags.dtype
                 )
-                nextindex = self._backend.index_nplike.full(
+                nextindex = self._backend.nplike.full(
                     len(carry_data),
                     self._contents[tag_for_missing].length,
                     dtype=self._index.dtype,
@@ -686,7 +700,7 @@ class UnionArray(UnionMeta[Content], Content):
             contents = []
             for tag, content in enumerate(self._contents):
                 if tag == tag_for_missing:
-                    indexedoption_index = self._backend.index_nplike.arange(
+                    indexedoption_index = self._backend.nplike.arange(
                         content.length + 1, dtype=np.int64
                     )
                     indexedoption_index[content.length] = -1
@@ -710,13 +724,13 @@ class UnionArray(UnionMeta[Content], Content):
         assert (
             self._index.length is unknown_length or lentags is unknown_length
         ) or self._index.length >= lentags
-        lenout = ak.index.Index64.empty(1, self._backend.index_nplike)
-        tmpcarry = ak.index.Index64.empty(lentags, self._backend.index_nplike)
+        lenout = ak.index.Index64.empty(1, self._backend.nplike)
+        tmpcarry = ak.index.Index64.empty(lentags, self._backend.nplike)
         assert (
-            lenout.nplike is self._backend.index_nplike
-            and tmpcarry.nplike is self._backend.index_nplike
-            and self._tags.nplike is self._backend.index_nplike
-            and self._index.nplike is self._backend.index_nplike
+            lenout.nplike is self._backend.nplike
+            and tmpcarry.nplike is self._backend.nplike
+            and self._tags.nplike is self._backend.nplike
+            and self._index.nplike is self._backend.nplike
         )
         self._backend.maybe_kernel_error(
             self._backend[
@@ -735,7 +749,7 @@ class UnionArray(UnionMeta[Content], Content):
             )
         )
         nextcarry = ak.index.Index64(
-            tmpcarry.data[: lenout[0]], nplike=self._backend.index_nplike
+            tmpcarry.data[: lenout[0]], nplike=self._backend.nplike
         )
         return self._contents[index]._carry(nextcarry, True)
 
@@ -746,13 +760,11 @@ class UnionArray(UnionMeta[Content], Content):
         backend: Backend,
         index_cls: type[Index] = Index64,
     ):
-        tags = tags.to_nplike(backend.index_nplike)
+        tags = tags.to_nplike(backend.nplike)
 
         lentags = tags.length
-        _size = ak.index.Index64.empty(1, nplike=backend.index_nplike)
-        assert (
-            _size.nplike is backend.index_nplike and tags.nplike is backend.index_nplike
-        )
+        _size = ak.index.Index64.empty(1, nplike=backend.nplike)
+        assert _size.nplike is backend.nplike and tags.nplike is backend.nplike
         backend.maybe_kernel_error(
             backend[
                 "awkward_UnionArray_regular_index_getsize",
@@ -764,13 +776,13 @@ class UnionArray(UnionMeta[Content], Content):
                 lentags,
             )
         )
-        size = backend.index_nplike.index_as_shape_item(_size[0])
-        current = index_cls.empty(size, nplike=backend.index_nplike)
-        outindex = index_cls.empty(lentags, nplike=backend.index_nplike)
+        size = backend.nplike.index_as_shape_item(_size[0])
+        current = index_cls.empty(size, nplike=backend.nplike)
+        outindex = index_cls.empty(lentags, nplike=backend.nplike)
         assert (
-            outindex.nplike is backend.index_nplike
-            and current.nplike is backend.index_nplike
-            and tags.nplike is backend.index_nplike
+            outindex.nplike is backend.nplike
+            and current.nplike is backend.nplike
+            and tags.nplike is backend.nplike
         )
         backend.maybe_kernel_error(
             backend[
@@ -802,25 +814,25 @@ class UnionArray(UnionMeta[Content], Content):
         tags_cls: type[Index] = Index8,
         index_cls: type[Index] = Index64,
     ) -> tuple[Index, Index]:
-        index_nplike = backend.index_nplike
+        nplike = backend.nplike
 
-        offsets = offsets.to_nplike(index_nplike)
-        counts = [c.to_nplike(index_nplike) for c in counts]
+        offsets = offsets.to_nplike(nplike)
+        counts = [c.to_nplike(nplike) for c in counts]
 
-        f_offsets = ak.index.Index64(index_nplike.asarray(offsets.data, copy=True))
-        contentlen = index_nplike.index_as_shape_item(
-            f_offsets[index_nplike.shape_item_as_index(f_offsets.length - 1)]
+        f_offsets = ak.index.Index64(nplike.asarray(offsets.data, copy=True))
+        contentlen = nplike.index_as_shape_item(
+            f_offsets[nplike.shape_item_as_index(f_offsets.length - 1)]
         )
 
-        tags = tags_cls.empty(contentlen, nplike=index_nplike)
-        index = index_cls.empty(contentlen, nplike=index_nplike)
+        tags = tags_cls.empty(contentlen, nplike=nplike)
+        index = index_cls.empty(contentlen, nplike=nplike)
 
         for tag, count in enumerate(counts):
             assert (
-                tags.nplike is index_nplike
-                and index.nplike is index_nplike
-                and f_offsets.nplike is index_nplike
-                and count.nplike is index_nplike
+                tags.nplike is nplike
+                and index.nplike is nplike
+                and f_offsets.nplike is nplike
+                and count.nplike is nplike
             )
             backend.maybe_kernel_error(
                 backend[
@@ -909,9 +921,7 @@ class UnionArray(UnionMeta[Content], Content):
 
         else:
             has_offsets = False
-            offsetsraws = self._backend.index_nplike.empty(
-                len(self._contents), dtype=np.intp
-            )
+            offsetsraws = self._backend.nplike.empty(len(self._contents), dtype=np.intp)
             contents = []
 
             keep_offsets = []
@@ -925,13 +935,11 @@ class UnionArray(UnionMeta[Content], Content):
                 has_offsets = offsets.length != 0
 
             if has_offsets:
-                total_length = ak.index.Index64.empty(
-                    1, nplike=self._backend.index_nplike
-                )
+                total_length = ak.index.Index64.empty(1, nplike=self._backend.nplike)
                 assert (
-                    total_length.nplike is self._backend.index_nplike
-                    and self._tags.nplike is self._backend.index_nplike
-                    and self._index.nplike is self._backend.index_nplike
+                    total_length.nplike is self._backend.nplike
+                    and self._tags.nplike is self._backend.nplike
+                    and self._index.nplike is self._backend.nplike
                 )
                 if self._backend.nplike == Numpy.instance():
                     self._backend.maybe_kernel_error(
@@ -969,21 +977,21 @@ class UnionArray(UnionMeta[Content], Content):
                     )
 
                 totags = ak.index.Index8.empty(
-                    total_length[0], nplike=self._backend.index_nplike
+                    total_length[0], nplike=self._backend.nplike
                 )
                 toindex = ak.index.Index64.empty(
-                    total_length[0], nplike=self._backend.index_nplike
+                    total_length[0], nplike=self._backend.nplike
                 )
                 tooffsets = ak.index.Index64.empty(
-                    self._tags.length + 1, nplike=self._backend.index_nplike
+                    self._tags.length + 1, nplike=self._backend.nplike
                 )
 
                 assert (
-                    totags.nplike is self._backend.index_nplike
-                    and toindex.nplike is self._backend.index_nplike
-                    and tooffsets.nplike is self._backend.index_nplike
-                    and self._tags.nplike is self._backend.index_nplike
-                    and self._index.nplike is self._backend.index_nplike
+                    totags.nplike is self._backend.nplike
+                    and toindex.nplike is self._backend.nplike
+                    and tooffsets.nplike is self._backend.nplike
+                    and self._tags.nplike is self._backend.nplike
+                    and self._index.nplike is self._backend.nplike
                 )
                 if self._backend.nplike == Numpy.instance():
                     self._backend.maybe_kernel_error(
@@ -1040,7 +1048,7 @@ class UnionArray(UnionMeta[Content], Content):
             else:
                 offsets = ak.index.Index64.zeros(
                     0,
-                    nplike=self._backend.index_nplike,
+                    nplike=self._backend.nplike,
                     dtype=np.int64,
                 )
                 return (
@@ -1078,17 +1086,17 @@ class UnionArray(UnionMeta[Content], Content):
 
         tags = ak.index.Index8.empty(
             theirlength + mylength,
-            nplike=self._backend.index_nplike,
+            nplike=self._backend.nplike,
         )
         index = ak.index.Index64.empty(
             theirlength + mylength,
-            nplike=self._backend.index_nplike,
+            nplike=self._backend.nplike,
         )
 
         contents = [other]
         contents.extend(self.contents)
 
-        assert tags.nplike is self._backend.index_nplike
+        assert tags.nplike is self._backend.nplike
         self._backend.maybe_kernel_error(
             self._backend["awkward_UnionArray_filltags_const", tags.dtype.type](
                 tags.data,
@@ -1098,7 +1106,7 @@ class UnionArray(UnionMeta[Content], Content):
             )
         )
 
-        assert index.nplike is self._backend.index_nplike
+        assert index.nplike is self._backend.nplike
         self._backend.maybe_kernel_error(
             self._backend["awkward_UnionArray_fillindex_count", index.dtype.type](
                 index.data,
@@ -1108,8 +1116,8 @@ class UnionArray(UnionMeta[Content], Content):
         )
 
         assert (
-            tags.nplike is self._backend.index_nplike
-            and self.tags.nplike is self._backend.index_nplike
+            tags.nplike is self._backend.nplike
+            and self.tags.nplike is self._backend.nplike
         )
         self._backend.maybe_kernel_error(
             self._backend[
@@ -1126,8 +1134,8 @@ class UnionArray(UnionMeta[Content], Content):
         )
 
         assert (
-            index.nplike is self._backend.index_nplike
-            and self.index.nplike is self._backend.index_nplike
+            index.nplike is self._backend.nplike
+            and self.index.nplike is self._backend.nplike
         )
         self._backend.maybe_kernel_error(
             self._backend[
@@ -1153,15 +1161,15 @@ class UnionArray(UnionMeta[Content], Content):
         if len(others) == 0:
             return self
 
-        index_nplike = self._backend.index_nplike
+        nplike = self._backend.nplike
         head, tail = self._merging_strategy(others)
 
         total_length = 0
         for array in head:
             total_length += array.length
 
-        nexttags = ak.index.Index8.empty(total_length, nplike=index_nplike)
-        nextindex = ak.index.Index64.empty(total_length, nplike=index_nplike)
+        nexttags = ak.index.Index8.empty(total_length, nplike=nplike)
+        nextindex = ak.index.Index64.empty(total_length, nplike=nplike)
 
         nextcontents = []
         length_so_far = 0
@@ -1176,10 +1184,7 @@ class UnionArray(UnionMeta[Content], Content):
                 union_tags = array.tags
                 union_index = array.index
                 union_contents = array.contents
-                assert (
-                    nexttags.nplike is index_nplike
-                    and union_tags.nplike is index_nplike
-                )
+                assert nexttags.nplike is nplike and union_tags.nplike is nplike
                 self._backend.maybe_kernel_error(
                     self._backend[
                         "awkward_UnionArray_filltags",
@@ -1193,10 +1198,7 @@ class UnionArray(UnionMeta[Content], Content):
                         len(nextcontents),
                     )
                 )
-                assert (
-                    nextindex.nplike is index_nplike
-                    and union_index.nplike is index_nplike
-                )
+                assert nextindex.nplike is nplike and union_index.nplike is nplike
                 self._backend.maybe_kernel_error(
                     self._backend[
                         "awkward_UnionArray_fillindex",
@@ -1219,7 +1221,7 @@ class UnionArray(UnionMeta[Content], Content):
                 isinstance(array, ak.contents.IndexedArray)
                 and array.parameter("__array__") != "categorical"
             ):
-                assert nexttags.nplike is index_nplike
+                assert nexttags.nplike is nplike
                 self._backend.maybe_kernel_error(
                     self._backend[
                         "awkward_UnionArray_filltags_const",
@@ -1232,9 +1234,9 @@ class UnionArray(UnionMeta[Content], Content):
                     )
                 )
                 nextindex.data[
-                    index_nplike.shape_item_as_index(
+                    nplike.shape_item_as_index(
                         length_so_far
-                    ) : index_nplike.shape_item_as_index(length_so_far + array.length)
+                    ) : nplike.shape_item_as_index(length_so_far + array.length)
                 ] = array.index.data
 
                 length_so_far += array.length
@@ -1247,7 +1249,7 @@ class UnionArray(UnionMeta[Content], Content):
                     )
                 )
             else:
-                assert nexttags.nplike is index_nplike
+                assert nexttags.nplike is nplike
                 self._backend.maybe_kernel_error(
                     self._backend[
                         "awkward_UnionArray_filltags_const",
@@ -1260,7 +1262,7 @@ class UnionArray(UnionMeta[Content], Content):
                     )
                 )
 
-                assert nextindex.nplike is index_nplike
+                assert nextindex.nplike is nplike
                 self._backend.maybe_kernel_error(
                     self._backend[
                         "awkward_UnionArray_fillindex_count", nextindex.dtype.type
@@ -1323,7 +1325,7 @@ class UnionArray(UnionMeta[Content], Content):
                         n, replacement, recordlookup, parameters, axis, depth
                     )
                 )
-            return ak.unionarray.UnionArray(
+            return ak.contents.UnionArray(
                 self._tags,
                 self._index,
                 contents,
@@ -1432,9 +1434,7 @@ class UnionArray(UnionMeta[Content], Content):
         if self._backend.nplike.known_data and self.index.length < self.tags.length:
             return f"at {path} ({type(self)!r}): len(index) < len(tags)"
 
-        lencontents = self._backend.index_nplike.empty(
-            len(self.contents), dtype=np.int64
-        )
+        lencontents = self._backend.nplike.empty(len(self.contents), dtype=np.int64)
         if self._backend.nplike.known_data:
             for j, _content_j in enumerate(self._contents):
                 lencontents[j] = _content_j.length
@@ -1452,7 +1452,7 @@ class UnionArray(UnionMeta[Content], Content):
             lencontents,
         )
 
-        if error.str is not None:
+        if error is not None and error.str is not None:
             if error.filename is None:
                 filename = ""
             else:
@@ -1672,7 +1672,7 @@ class UnionArray(UnionMeta[Content], Content):
             raise AssertionError(result)
 
     def to_packed(self, recursive: bool = True) -> Self:
-        index_nplike = self._backend.index_nplike
+        nplike = self._backend.nplike
         tags = self._tags.data
         original_index = index = self._index.data[: tags.shape[0]]
 
@@ -1680,16 +1680,18 @@ class UnionArray(UnionMeta[Content], Content):
 
         for tag in range(len(self._contents)):
             is_tag = tags == tag
-            num_tag = index_nplike.index_as_shape_item(
-                index_nplike.count_nonzero(is_tag)
-            )
+            num_tag = nplike.index_as_shape_item(nplike.count_nonzero(is_tag))
 
             if len(contents[tag]) > num_tag:
                 if original_index is index:
                     index = index.copy()
-                index[is_tag] = self._backend.index_nplike.arange(
+                new_index_values = self._backend.nplike.arange(
                     num_tag, dtype=index.dtype
                 )
+                if isinstance(self._backend.nplike, Jax):
+                    index = index.at[is_tag].set(new_index_values)
+                else:
+                    index[is_tag] = new_index_values
                 contents[tag] = self.project(tag)
 
             contents[tag] = (
@@ -1697,8 +1699,8 @@ class UnionArray(UnionMeta[Content], Content):
             )
 
         return UnionArray(
-            ak.index.Index8(tags, nplike=self._backend.index_nplike),
-            ak.index.Index(index, nplike=self._backend.index_nplike),
+            ak.index.Index8(tags, nplike=self._backend.nplike),
+            ak.index.Index(index, nplike=self._backend.nplike),
             contents,
             parameters=self._parameters,
         )
@@ -1721,8 +1723,8 @@ class UnionArray(UnionMeta[Content], Content):
         return out
 
     def _to_backend(self, backend: Backend) -> Self:
-        tags = self._tags.to_nplike(backend.index_nplike)
-        index = self._index.to_nplike(backend.index_nplike)
+        tags = self._tags.to_nplike(backend.nplike)
+        index = self._index.to_nplike(backend.nplike)
         contents = [content.to_backend(backend) for content in self._contents]
         return UnionArray(
             tags,
